#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 20:46:21 2022

@author: mehlmann
"""

import numpy as np
import matplotlib.pyplot as plt

# Implementation of the right hand side 
# x'(t) = f( x(t) )
def f(x):
    return np.array([x[1],
                     -x[0]-0.01*x[1]-100*x[2],
                     x[1]-1000.0*x[2]])


## One step of Heun's 2nd order method
def heun(x,h):
    return x+0.5*h*f(x) + 0.5*h*f(x+h*f(x))


## Solve the problem in [0,10] and plot the solution
T = 10   # Final time
N = 6000  # Number of time steps
h = T/N  # step size

timesteps = np.linspace(0,T,N+1)  # discrete time points

x = np.zeros( (N+1,3) )           # solution vector
x[0,:] = np.array([1,1,0])        # initial solution

for i in range(N):
    x[i+1,:] = heun(x[i,:],h)
    
print('\n(i) With Heun''s method using h={0}: x(10)='.format(h),x[-1])

