#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 20:46:21 2022

@author: mehlmann
"""

import numpy as np
import matplotlib.pyplot as plt

# we solve the model problem
#
#  u' = lambda u, u(0) = 1
#
# lambda is a complex number and for z = h * lambda we determine
# if the methods are stable by analyzing one single step
# if |u_1|>1 the method is unstable, otherwise it is stable

# One step of E Euler
# for u' = lambda u
# u[n+1] = u[n] + h * lambda * u[n]
#        = u[n] + z * u[n]
def eeuler(x,z):
    return x+z*x



## We create a mesh of points x/y from -4 to 4, real- and imaginary part
N = 200
lx = np.linspace(-4,4,N)
X,Y = np.meshgrid(lx,lx)  # X[i,j] and Y[i,j] are the real- and imaginart parts


EE = np.zeros( (N,N) )  # We store the solution for one step of explicit euler

for x in range(N): # real part
    for y in range(N): # imag part
    
        # Numpy can work with complex numbers. 
        # 1.j is the imaginary unit.
        z = lx[x]+lx[y]*1.j          
        
        if np.absolute(eeuler(1.,z))<=1:  #  check, if the first step is stable
            EE[y,x]=1
 

### scatter plots points on a surface
plt.scatter(X,Y,EE,label='forward Euler')
plt.plot([0,0],[-4,4],'gray')
plt.plot([-4,4],[0,0],'gray')
plt.legend()
plt.show()

