#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 20:46:21 2022

@author: mehlmann
"""

import numpy as np
import matplotlib.pyplot as plt


# The matrix of the equation
# x'(t) = A x(t)
def A():
    return np.array([[0,1,0],
                     [-1,-0.01,-100],
                     [0,1,-1000]])


## One step of forward Euler
## [I - h A] u_n = u_{n-1}
def ieuler(x,h):
    
    # np.eye(3) is the identity matrix I
    # np.linalg.inv(np.eye(3)-h*A()) is the inverse of [I-hA]
    # np.dot is the matrix-vector product
    return np.dot(np.linalg.inv(np.eye(3)-h*A()),x)


print('3. Implicit discretizations of the linear equation.\n')

print('(i) backward Euler solution on [0,T]')
## Solve the problem in [0,10] and plot the solution
T = 10   # Final time
N = 10  # Number of time steps
h = T/N  # step size

timesteps = np.linspace(0,T,N+1)  # discrete time points

x = np.zeros( (N+1,3) )           # solution vector
x[0,:] = np.array([1,1,0])        # initial solution

for i in range(N):
    x[i+1,:] = ieuler(x[i,:],h)
    
plt.plot(timesteps,x,label='Implicit Euler')
plt.legend()
plt.show()

print('The backward Euler solution with h={} is x(10)='.format(h),x[-1,:])



