#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 20:46:21 2022

@author: mehlmann
"""

import numpy as np
import matplotlib.pyplot as plt

# Implementation of the right hand side 
# x'(t) = f( x(t) )
def f(x):
    return np.array([x[1],
                     -x[0]-0.01*x[1]-100*x[1]*x[1]*x[2],
                     np.exp(2*x[1])-1000.0*x[2]])


## Implement the Jacobian Df(x)
def Df(x):
    return np.array([[0,1,0],
                     [-1, -0.01-200.0*x[1]*x[2],-100*x[1]*x[1]],
                     [0,np.exp(2*x[1])*2.,-1000]])


## One step of Implicity Euler with Newton
# x - h f(x) = o
# Newton:
#    [I - h Df(x)] w = o - (x - h f(x)) 
#    x = x + w 
def ieuler(x,h):
    xold = x
    for i in range(5): # We do 5 steps of newton
        w = np.dot(np.linalg.inv(np.eye(3)-h*Df(x)),xold - (x-h*f(x)))
        x = x + w
    return x


## One step of Heun's 2nd order method
def heun(x,h):
    return x+0.5*h*f(x) + 0.5*h*f(x+h*f(x))

## Solve the problem in [0,10] and plot the solution
T = 10   # Final time
N = 6000  # Number of time steps
h = T/N  # step size
timesteps = np.linspace(0,T,N+1)  # discrete time points

x = np.zeros( (N+1,3) )           # solution vector
x[0,:] = np.array([1,1,0])        # initial solution

for i in range(N):
    x[i+1,:] = heun(x[i,:],h)

plt.title('Nonlinear spring pendulum')    
plt.plot(timesteps,x,label='Heun''s solution')


## Solve the problem in [0,10] and plot the solution
N = 50  # Number of time steps
h = T/N  # step size
timesteps = np.linspace(0,T,N+1)  # discrete time points
x = np.zeros( (N+1,3) )           # solution vector
x[0,:] = np.array([1,1,0])        # initial solution
for i in range(N):
    x[i+1,:] = ieuler(x[i,:],h)

plt.plot(timesteps,x,'-*',label='Implicit Euler')

plt.legend()
plt.show()
